@@ -1,7 +1,8 @@
+;;; -*- lexical-binding: nil; -*-
 ;;; action-lock.el --- invoke magic action by RET key on spell strings
 
-;; Copyright (C) 2002, 2003, 2004, 2005-2022
-;;   HIRAOKA Kazuyuki <khi@users.osdn.me>
+;; Copyright (C) 2002, 2003, 2004, 2005-2025
+;;   HIRAOKA Kazuyuki <kakkokakko@gmail.com>
 ;;
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -53,8 +54,6 @@
 (defvar action-lock-magic-return-key "\C-m")
 (put 'action-lock-magic-return-key 'risky-local-variable t)
 (defvar action-lock-lighter " AL")
-(defvar action-lock-silent t
-  "Inhibit font-lock-verbose if non-nil.")
 
 ;; If you want to change these values,
 ;; you must set them before loading this file.
@@ -62,11 +61,11 @@
 (defvar action-lock-date-default '("{_}" "[%Y-%m-%d %H:%M]"))  ;; before after
 
 (define-minor-mode action-lock-mode
-  "With no argument, this command toggles the mode.
-Non-null prefix argument turns on the mode.
-Null prefix argument turns off the mode.
+  "Toggle Action Lock mode.
 
-\\[action-lock-magic-return]  Envoke the action on the field
+key	binding
+---	-------
+\\<action-lock-mode-map>\\[action-lock-magic-return]	Invoke the action on the field
 "
   :init-value nil ;; default = off
   :lighter action-lock-lighter ;; mode-line
@@ -264,7 +263,7 @@ at the beginning of this file, when ARG-P is non-nil."
     regexp))
 
 (defun action-lock-font-lock ()
-  (cheat-font-lock-mode action-lock-silent)
+  (cheat-font-lock-mode)
   (if (null action-lock-original-font-lock-keywords)
       (setq action-lock-original-font-lock-keywords font-lock-keywords)
     (setq font-lock-keywords action-lock-original-font-lock-keywords))
@@ -278,8 +277,7 @@ at the beginning of this file, when ARG-P is non-nil."
                                 (cons matcher hilit)))
                             action-lock-rules)))
       (cheat-font-lock-append-keywords entries)
-;;       (cheat-font-lock-prepend-keywords entries)
-      (cheat-font-lock-fontify t)
+      (cheat-font-lock-fontify)
       )))
 
 (defun action-lock-restore-font-lock ()
