@@ -1,4 +1,5 @@
 #!/usr/bin/ruby -s
+# -*- coding: euc-jp -*-
 # -*- Ruby -*-
 
 def usage
@@ -6,8 +7,8 @@ def usage
   print <<EOU
 #{name}: howm , եʬ (ȴ)
 ()
-  #{name} 2004_10_10.howm
-   2004_10_10.howm.aa, 2004_10_10.howm.ab,  Ǥ
+  #{name} 2004_10_10.txt
+   2004_10_10.txt.aa, 2004_10_10.txt.ab,  Ǥ
 (ץ)
   -prefix=hoge.      hoge.aa, hoge.ab,  Ǥ
   -help ޤ -h    Υåɽ
