@@ -1,6 +1,7 @@
+;;; -*- lexical-binding: nil; -*-
 ;;; howm-date.el --- Wiki-like note-taking tool
-;;; Copyright (C) 2002, 2003, 2004, 2005-2021
-;;;   HIRAOKA Kazuyuki <khi@users.osdn.me>
+;;; Copyright (C) 2002, 2003, 2004, 2005-2025
+;;;   HIRAOKA Kazuyuki <kakkokakko@gmail.com>
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -41,7 +42,7 @@
 (defun howm-action-lock-date (date &optional new future-p)
   (let* ((pass-through (and new howm-insert-date-pass-through))
          (prompt (howm-action-lock-date-prompt date new pass-through))
-         (immediate-chars (if pass-through "" "."))
+         (immediate-chars (if pass-through "[]" ".[]"))
          (c (howm-read-string prompt immediate-chars "+-~0123456789"
                               pass-through pass-through)))
     (cond
@@ -59,33 +60,52 @@
       (howm-action-lock-date-repeat (match-string-no-properties 1 c) date))
      ((string-match "^[.]$" c)
       (howm-datestr-replace (howm-time-to-datestr)))
+     ((string= c "[") (howm-action-lock-date-quickshift -1 date))
+     ((string= c "]") (howm-action-lock-date-quickshift +1 date))
      ((and (string-match "^[-+~]$" c) pass-through)
       (insert c))
      (t (error (format "Can't understand %s." c))))))
 
+(defun howm-action-lock-date-quickshift (n date)
+  ;; We should provide an official way to exit quick-shift other than
+  ;; C-g, ensuring the list is updated when we hit RET on dates in the
+  ;; todo list.
+  ;; See howm-action-lock-forward in howm-reminder.el.
+  (let* ((new-date (howm-action-lock-date-shift n date))
+         (dow (howm-datestr-day-of-week new-date))
+         (prompt (format "%s [%s] RET(ok), [(-1), ](+1), .(today): " new-date dow))
+         (c (howm-read-string prompt "[]." nil t nil)))
+    (cond
+     ((string= c "[") (howm-action-lock-date-quickshift -1 new-date))
+     ((string= c "]") (howm-action-lock-date-quickshift +1 new-date))
+     ((string= c ".")
+      (howm-action-lock-date-quickshift +0 (howm-time-to-datestr))))))
+
 (defun howm-action-lock-date-prompt (date new pass-through)
   (let* ((dow (howm-datestr-day-of-week date))
          (common-help "+num(shift), yymmdd(set), ~yymmdd(repeat)")
          (today-help ", .(today)")
+         (shift-help ", [(-1), ](+1)")
          (help (cond ((and new pass-through)
-                      common-help)
+                      (concat common-help shift-help))
                      ((and new (not pass-through))
-                      (concat "RET(ok), " common-help today-help))
+                      (concat "RET(ok), " common-help today-help shift-help))
                      ((not new)
-                      (concat "RET(list), " common-help today-help))
+                      (concat "RET(list), " common-help today-help shift-help))
                      (t
                       (error "Can't happen.")))))
-    (format "[%s] %s: " dow help)))
+    (format "%s [%s] %s: " date dow help)))
 
 (defvar howm-date-current nil)
 (make-variable-buffer-local 'howm-date-current)
 
 (defun howm-action-lock-date-search (date)
   (howm-set-command 'howm-action-lock-date-search)
-  (prog1
-      (howm-search date t)
+  (let ((items (howm-search date t)))
     (howm-action-lock-forward-escape)
-    (setq howm-date-current date)))
+    (when items
+      (setq howm-date-current date))
+    items))
 
 (defun howm-search-today ()
   (interactive)
@@ -99,7 +119,9 @@
     (howm-action-lock-date-search target)))
 
 (defun howm-action-lock-date-shift (n date)
-  (howm-datestr-replace (howm-datestr-shift date 0 0 n)))
+  (let ((new-date (howm-datestr-shift date 0 0 n)))
+    (howm-datestr-replace new-date)
+    new-date))
 
 (defun howm-action-lock-date-set (val date &optional future-p)
   (howm-datestr-replace (howm-datestr-expand val date future-p)))
@@ -235,14 +257,14 @@
          (c 0))
     (when (catch :found
             (while (progn
-                   (when (howm-action-lock-date-search new-date)
-                     (throw :found t))
-                   (< c howm-date-forward-ymd-limit))
-            (setq new-date (howm-datestr-shift new-date 0 0 step))
-            (setq c (1+ c))
-            (when howm-date-forward-ymd-msg
-              (format howm-date-forward-ymd-msg new-date)))
-          (error "Not found within %d days." howm-date-forward-ymd-limit))
+                     (when (howm-action-lock-date-search new-date)
+                       (throw :found t))
+                     (< c howm-date-forward-ymd-limit))
+              (setq new-date (howm-datestr-shift new-date 0 0 step))
+              (setq c (1+ c))
+              (when howm-date-forward-ymd-msg
+                (message howm-date-forward-ymd-msg new-date)))
+            (error "Not found within %d days." howm-date-forward-ymd-limit))
       (when (not (eq (current-buffer) b))
         (with-current-buffer b
           (howm-view-kill-buffer)))
