@@ -1,6 +1,7 @@
+;;; -*- lexical-binding: nil; -*-
 ;;; howm-lang-en.el --- Wiki-like note-taking tool
-;;; Copyright (C) 2005-2021
-;;;   HIRAOKA Kazuyuki <khi@users.osdn.me>
+;;; Copyright (C) 2005-2025
+;;;   HIRAOKA Kazuyuki <kakkokakko@gmail.com>
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -28,7 +29,7 @@
     ("[New]" (lambda () (howm-create ,howm-menu-action-arg)))
     ("[Add]" (lambda () (howm-create-here ,howm-menu-action-arg)))
     ("[Dup]" howm-dup)
-    ("[Update]" howm-initialize-buffer previous)
+    ("[Update]" howm-menu-refresh-note previous)
     ("[Regexp]" howm-list-grep)
     ("[String]" howm-list-grep-fixed)
     ;;         ("[roma]" howm-list-migemo)
@@ -56,6 +57,25 @@
     ("[Random Walk]" howm-random-walk previous)
     ))
 
+;; based on https://github.com/kaorahi/howm/issues/57
+(defun howm-menu-legend-en ()
+  (cl-labels ((p (text face) (propertize text 'font-lock-face face)))
+    (concat
+     ;; first line
+     "[Schedule, Todo] -- "
+     (p "@schedule" howm-reminder-schedule-face) ", "
+     (p "!deadline" howm-reminder-deadline-face) " "
+     (format "(until %s days from now)" howm-menu-schedule-days) "\n"
+     ;; second line
+     (p "!deadline" howm-reminder-deadline-face) ", "
+     (p "+todo" howm-reminder-todo-face) ", "
+     (p "-reminder" howm-reminder-normal-face) ", "
+     (p "~defer" howm-reminder-defer-face) " "
+     (format "(top %s entries)" howm-menu-todo-num) " "
+     "[" (p "today" howm-reminder-today-face) "] "
+     "[" (p "tomorrow" howm-reminder-tomorrow-face) "]"
+     )))
+
 (provide 'howm-lang-en)
 
 ;;; howm-lang-en.el ends here
