@@ -1,6 +1,7 @@
+;;; -*- lexical-binding: nil; Coding: utf-8-unix -*-
 ;;; howm-menu.el --- Wiki-like note-taking tool
-;;; Copyright (C) 2002, 2003, 2004, 2005-2022
-;;;   HIRAOKA Kazuyuki <khi@users.osdn.me>
+;;; Copyright (C) 2002, 2003, 2004, 2005-2025
+;;;   HIRAOKA Kazuyuki <kakkokakko@gmail.com>
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -35,7 +36,7 @@
   (define-key m "\M-\C-i" 'action-lock-goto-previous-link)
   (define-key m " " 'scroll-up)
   (define-key m [backspace] 'scroll-down)
-  (define-key m "\C-h" 'scroll-down)
+  (define-key m (kbd "DEL") 'scroll-down)
   (define-key m "q" 'bury-buffer)
   (define-key m "?" 'describe-mode)
   (setq howm-menu-mode-map m)
@@ -90,6 +91,10 @@ in `howm-menu-mode'.")
     howm-menu-random
     howm-menu-search
     howm-menu-categorized-reminder
+    howm-menu-last-update
+    howm-menu-legend-en
+    howm-menu-legend-fr
+    howm-menu-legend-ja
     ))
 
 (howm-defvar-risky howm-menu-display-rules
@@ -102,6 +107,7 @@ in `howm-menu-mode'.")
     ("%reminder" . "%here%(howm-menu-reminder)")
     ("%recent"   . "%here%(howm-menu-recent)")
     ("%random"   . "%here%(howm-menu-random)")
+    ("%last-update" . "%here%(howm-menu-last-update)")
     ;; dynamic
     ("%here%" . howm-menu-here)
     (,howm-menu-key-regexp . howm-menu-shortcut)
@@ -166,10 +172,16 @@ Regexp R1 is replaced by T1 if T1 is a string.
 
 (defun howm-menu-mode ()
   "howm menu
+
+Think of this menu as a \"modal dialog\" rather than a workspace.
+Typically, the commands listed in the menu apply to the buffer BEHIND
+the menu, not the menu buffer itself.  In other words, they affect the
+buffer that was active before the menu appeared.
+
 key	binding
 ---	-------
-\\[action-lock-magic-return]	Follow link
-\\[action-lock-goto-next-link]	Next link
+\\<action-lock-mode-map>\\[action-lock-magic-return]	Follow link
+\\<howm-menu-mode-map>\\[action-lock-goto-next-link]	Next link
 \\[action-lock-goto-previous-link]	Prev link
 \\[describe-mode]	This help
 \\[bury-buffer]	Quit
@@ -239,10 +251,17 @@ key	binding
 (howm-defvar-risky howm-menu-shortcut-assoc nil)
 (make-variable-buffer-local 'howm-menu-shortcut-assoc)
 (howm-defvar-risky howm-menu-invisible t
-  "*Non nil if 'invisible' property should be used in menu.
+  "*Non nil if \"invisible\" property should be used in menu.
 This must be t at now.
 When this is nil, delete-region is used instead, and bug appears.")
 
+(defun howm-menu-refresh-note ()
+  (unless howm-mode
+    ;; warn about common misuse
+    (error "Not howm-mode. Press ? in the howm menu to see why."))
+  (howm-initialize-buffer)
+  (message "%s has been updated." (buffer-name)))
+
 (defun howm-menu-refresh (&optional file place name)
   (interactive)
   ;; preprocess
@@ -254,6 +273,7 @@ When this is nil, delete-region is used instead, and bug appears.")
                                         howm-menu-buffer-file-place
                                         1))
   (setq howm-menu-shortcut-assoc nil)
+  (setq howm-menu-last-time (current-time))
   ;; main
   (howm-rewrite-read-only-buffer
     (howm-menu-insert-paragraph howm-menu-buffer-file
@@ -262,7 +282,6 @@ When this is nil, delete-region is used instead, and bug appears.")
     (howm-menu-set-face))
   ;; postprocess
   (goto-char (point-min))
-  (setq howm-menu-last-time (current-time))
   (setq howm-menu-next-expiry-time
         (howm-days-after (current-time) 0
                          howm-menu-expiry-hours))
@@ -300,7 +319,8 @@ When this is nil, delete-region is used instead, and bug appears.")
 (defun howm-menu-set-face ()
   (set (make-local-variable 'font-lock-keywords-only) t)
   (howm-menu-add-font-lock)
-  (font-lock-fontify-buffer)
+  (font-lock-flush)
+  (font-lock-ensure)
   (when howm-menu-toggle-invisible
     (howm-menu-make-invisible)))
 
@@ -324,6 +344,11 @@ When this is nil, delete-region is used instead, and bug appears.")
     (howm-menu t)
     (switch-to-buffer b)))
 
+;; https://github.com/kaorahi/howm/issues/44#issuecomment-2648850457
+(defun howm-menu-last-update ()
+  "Return when the menu was last updated."
+  (format-time-string howm-dtime-format howm-menu-last-time))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; action-lock
 
@@ -524,10 +549,14 @@ When this is nil, delete-region is used instead, and bug appears.")
 
 (defun howm-find-today (&optional days-before)
   (interactive "P")
-  (howm-find-past (or days-before 0)))
+  (let ((d (cond ((null days-before) 0)
+                 ((listp days-before) (car days-before))
+                 ((eq '- days-before) -1)
+                 (t days-before))))
+    (howm-find-past d)))
 
 (defun howm-find-yesterday (&optional days-before)
-  (interactive)
+  (interactive "p")
   (howm-find-past (or days-before 1)))
 
 (defun howm-one-file-one-day-p ()
@@ -665,8 +694,8 @@ When this is nil, delete-region is used instead, and bug appears.")
 LABEL is only used for message.
 FORMATTER is a function which receives an item and returns an output string
  (without newline).
-FORMATTER can be nil for standard style, 'todo for todo style,
-'schedule for schedule style, or 'full for full note.
+FORMATTER can be nil for standard style, \\='todo for todo style,
+\\='schedule for schedule style, or \\='full for full note.
 ITEM-LIST is list of items which should be shown."
   (let ((f (cond ((null formatter) #'howm-menu-format-item)
                  ((eq 'todo formatter) #'howm-menu-format-todo)
@@ -740,17 +769,23 @@ ITEM-LIST is list of items which should be shown."
          (sorted (howm-sort (lambda (f) (funcall h-r-m-evaluator f))
                             #'howm-view-string>
                             (mapcar #'howm-item-name
-                                    (howm-folder-items howm-directory t))))
+                                    (howm-recent-items-filter
+                                     (howm-folder-items
+                                      howm-directory t)))))
          (files (howm-first-n sorted num)))
-    (let ((r (howm-menu-recent-regexp)))
+    (let* ((r (howm-menu-recent-regexp))
+           (searcher (lambda (f)
+                       (let* ((is (or (howm-view-search-items r (list f) summarizer)
+                                      (list (howm-make-item f)))))
+                         (mapc #'howm-view-change-title is)
+                         is))))
       (if randomp
           (cl-mapcan (lambda (f)
-                            (let ((is (howm-view-search-items r (list f)
-                                                              summarizer)))
-                              (and is (list (nth (random (length is))
-                                                 is)))))
-                          files)
-        (howm-first-n (howm-view-search-items r files summarizer) num)))))
+                       (let ((is (funcall searcher f)))
+                         (and is (list (nth (random (length is))
+                                            is)))))
+                     files)
+        (howm-first-n (cl-mapcan searcher files) num)))))
 
 (defun howm-menu-recent-regexp ()
   (or howm-menu-recent-regexp (howm-view-title-regexp-grep)))
@@ -887,11 +922,18 @@ If you don't like misc. category, try
     (howm-view-kill-buffer)
     (howm-menu nil t)))
 
+(defvar howm-menu-skel-replace-rules
+  `(
+    ("^= " . ,(format "%s " howm-view-title-header))
+    ))
+
+(defvar howm-menu-file-extension ".txt")
+
 (defun howm-menu-copy-skel (contents)
-  (let ((menu-file (or howm-menu-file
-                       (expand-file-name "0000-00-00-000000.txt"
-                                         howm-directory)))
-        (r "^="))
+  (let* ((default-name (format "0000-00-00-000000%s" howm-menu-file-extension))
+         (menu-file (or howm-menu-file
+                        (expand-file-name default-name
+                                          howm-directory))))
     (if (file-exists-p menu-file)
         ;; I have no courage to erase existing file.
         (progn
@@ -900,9 +942,7 @@ If you don't like misc. category, try
       (progn
         (find-file menu-file)
         (insert contents)
-        (goto-char (point-min))
-        (while (re-search-forward r nil t)
-          (replace-match howm-view-title-header nil nil))
+        (howm-menu-replace howm-menu-skel-replace-rules)
         (howm-mode 1)
         (save-buffer)))))
 
@@ -937,8 +977,18 @@ If you don't like misc. category, try
   (string= major-mode "howm-menu-mode"))
 
 (defun howm-menu-name (file)
+  (howm-menu-name-avoid-invisible)
   (format howm-menu-name-format file))
 
+(defun howm-menu-name-avoid-invisible ()
+  ;; invisible buffers are no longer supported by cheat-font-lock
+  (when (string-match "^ +\\(.*\\)" howm-menu-name-format)
+    (let ((orig howm-menu-name-format)
+          (fixed (match-string 1 howm-menu-name-format)))
+      (setq howm-menu-name-format fixed)
+      (message "howm-menu-name-format is changed from \"%s\" to \"%s\" by howm-menu-name-avoid-invisible"
+               orig fixed))))
+
 (defun howm-buffer-alive-p (buf)
   (and buf (buffer-name buf)))
 
