@@ -1,6 +1,7 @@
+;;; -*- lexical-binding: nil; -*-
 ;;; howm-misc.el --- Wiki-like note-taking tool
-;;; Copyright (C) 2002, 2003, 2004, 2005-2021
-;;;   HIRAOKA Kazuyuki <khi@users.osdn.me>
+;;; Copyright (C) 2002, 2003, 2004, 2005-2025
+;;;   HIRAOKA Kazuyuki <kakkokakko@gmail.com>
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -404,8 +405,9 @@ and replace a sub-expression, e.g.
        (,howm-keyword-regexp-hilit-pos howm-mode-keyword-face prepend))
       (,howm-ref-regexp
        (,howm-ref-regexp-hilit-pos howm-mode-ref-face prepend))
-      (,howm-wiki-regexp
-       (,howm-wiki-regexp-pos howm-mode-wiki-face prepend))
+      ,@(and howm-wiki-regexp
+             `((,howm-wiki-regexp
+                (,howm-wiki-regexp-pos howm-mode-wiki-face prepend))))
       )))
 
 ;;; unofficial. may be removed if no one needs them.
@@ -427,10 +429,12 @@ and replace a sub-expression, e.g.
 ;;; narrowing
 
 (defun howm-narrow-to-memo ()
+  "Restrict editing in this buffer to the current memo."
   (interactive)
   (apply #'narrow-to-region (howm-view-paragraph-region t)))
 
 (defun howm-toggle-narrow ()
+  "Toggle whether to restrict editing in this buffer to the current memo."
   (interactive)
   (if (howm-narrow-p)
       (widen)
@@ -617,7 +621,7 @@ and replace a sub-expression, e.g.
            (s (buffer-substring-no-properties (point-min) (point-max))))
       (set-window-configuration howm-remember-wconf)
       (howm-create-file-with-title title)
-      (insert s "\n")
+      (insert (format howm-remember-insertion-format s))
       (save-buffer)
       (kill-buffer (current-buffer))))
   (howm-remember-discard))
@@ -700,6 +704,38 @@ When DOTS-STR is non-nil, it is used instead of \"...\"."
                                    howm-view-title-regexp
                                    howm-view-title-regexp-pos))
 
+;; Automatic theming
+;; https://github.com/kaorahi/howm/issues/34
+
+(defun howm-auto-theme (&rest _)
+  ;; copied from https://github.com/kaorahi/howm/issues/34#issue-2815035530
+  "Adjust to the current theme by borrowing the Org-mode colors."
+  (when custom-enabled-themes
+    (custom-set-faces
+     '(action-lock-face ((t :inherit button)))
+     '(howm-mode-keyword-face ((t :inherit org-link)))
+     '(howm-mode-ref-face ((t :inherit org-link)))
+     '(howm-mode-title-face ((t :inherit org-level-1)))
+     '(howm-mode-wiki-face ((t :inherit org-link)))
+     '(howm-reminder-deadline-face ((t :inherit org-scheduled-today)))
+     '(howm-reminder-late-deadline-face ((t :inherit bold :inherit org-imminent-deadline)))
+     '(howm-reminder-defer-face ((t :inherit org-scheduled)))
+     '(howm-reminder-scheduled-face ((t :inherit org-scheduled)))
+     '(howm-reminder-done-face ((t :inherit org-done)))
+     '(howm-reminder-todo-face ((t :inherit org-todo)))
+     '(howm-reminder-normal-face ((t :inherit org-default)))
+     '(howm-reminder-today-face ((t :inherit bold :inherit org-scheduled-today)))
+     '(howm-reminder-tomorrow-face ((t :inherit bold :inherit org-scheduled)))
+     '(howm-simulate-todo-mode-line-face ((t :inherit bold)))
+     '(howm-view-empty-face ((t :inherit shadow)))
+     '(howm-view-hilit-face ((t :inherit isearch)))
+     '(howm-view-name-face ((t :inherit org-document-info))))
+    (apply #'custom-set-faces howm-auto-theme-custom-entries)))
+
+(when howm-follow-theme
+  (add-hook 'enable-theme-functions #'howm-auto-theme)
+  (howm-auto-theme))
+
 ;; memoize: used in howm-bayesian-set
 
 (defun howm-memoize-put (fname value)
@@ -855,20 +891,20 @@ When DOTS-STR is non-nil, it is used instead of \"...\"."
 ;; xemacs canna doesn't use minor-mode. [2004-01-30]
 (defvar action-lock-mode-before-canna nil)
 (make-variable-buffer-local 'action-lock-mode-before-canna)
-(defadvice canna:enter-canna-mode (around action-lock-fix activate)
+(define-advice canna:enter-canna-mode (:around (orig-fun &rest args) action-lock-fix)
   (setq action-lock-mode-before-canna action-lock-mode)
   (setq action-lock-mode nil)
-  ad-do-it)
-(defadvice canna:quit-canna-mode (around action-lock-fix activate)
+  (apply orig-fun args))
+(define-advice canna:quit-canna-mode (:around (orig-fun &rest args) action-lock-fix)
   (setq action-lock-mode action-lock-mode-before-canna)
-  ad-do-it)
+  (apply orig-fun args))
  
 ;; for mcomplete.el [2003-12-17]
 ;; http://homepage1.nifty.com/bmonkey/emacs/elisp/mcomplete.el
 ;; error when this-command is (lambda () (interactive) ...)
-(defadvice mcomplete-p (around symbol-check activate)
+(define-advice mcomplete-p (:around (orig-fun &rest args) symbol-check)
   (and (symbolp this-command)
-       ad-do-it))
+       (apply orig-fun args)))
 
 ;; for auto-save-buffers.el [2004-01-10]
 ;; http://www.namazu.org/~satoru/auto-save/
@@ -890,37 +926,36 @@ When DOTS-STR is non-nil, it is used instead of \"...\"."
   (setq howm-refresh-after-save nil)
   (setq howm-auto-save-buffers-disposed t)
   (message "howm: Automatic refresh is disabled when auto-save-buffers is called."))
-(defadvice auto-save-buffers (around howm-dispose activate)
+(define-advice auto-save-buffers (:around (orig-fun &rest args) howm-dispose)
   (if (or howm-auto-save-buffers-disposed
           (not (howm-auto-save-buffers-p)))
-      ad-do-it
+      (apply orig-fun args)
     (howm-auto-save-buffers-dispose)))
 
 ;; howm on ChangeLog Memo
 (defun howm-setup-change-log ()
   (setq howm-keyword-format "\t* %s")
   (setq howm-keyword-regexp "^\t\\(\\*\\)[ \t]+\\([^:\r\n]+\\)")
-  (setq howm-keyword-regexp-hilit-pos 1) ;; ִϢɡ
+  (setq howm-keyword-regexp-hilit-pos 1) ;; for "related keywords"
   (setq howm-keyword-regexp-pos 2)
   (setq howm-view-title-regexp "^$")
   (setq howm-view-title-regexp-pos 0)
   (setq howm-view-title-regexp-grep 'sorry-not-yet)
   (setq howm-use-color nil)
   (setq howm-menu-top nil)
-  (defadvice howm-exclude-p (around change-log (filename) activate)
-    (setq ad-return-value
-          (not (find-if (lambda (dir)
-                          (string= (howm-file-name)
-                                   (file-relative-name filename dir)))
-                        (howm-search-path)))))
-  (defadvice howm-create-file-with-title (around change-log (title) activate)
+  (define-advice howm-exclude-p (:around (orig-fun filename) change-log)
+          (not (cl-find-if (lambda (dir)
+                             (string= (howm-file-name)
+                                      (file-relative-name filename dir)))
+                           (howm-search-path))))
+  (define-advice howm-create-file-with-title (:around (orig-fun title) change-log)
     (howm-create-file)
     (when (string-match howm-keyword-regexp title)
       (setq title (match-string-no-properties howm-keyword-regexp-pos
                                               title)))
     (insert title))
-  (defadvice howm-create-file (around change-log
-                                      (&optional keep-cursor-p) activate)
+  (define-advice howm-create-file
+      (:around (orig-fun title &optional keep-cursor-p) change-log)
     (let* ((default (howm-file-name))
            (file (expand-file-name default howm-directory))
            (dir (file-name-directory file))
@@ -948,6 +983,32 @@ When DOTS-STR is non-nil, it is used instead of \"...\"."
                                              title-regexp title-regexp-pos)))
     (howm-create-file-with-title title)))
 
+(define-minor-mode howm-org-font-lock-minor-mode
+  ;; copied from https://github.com/kaorahi/howm/issues/29#issuecomment-2625076294
+  ;; (fixed version of https://github.com/kaorahi/howm/issues/29#issuecomment-2571306675)
+  "Minor mode to apply org-mode font locking to a buffer."
+  :lighter " orgFL"
+  (if howm-org-font-lock-minor-mode
+      (progn
+        ;; [2025-05-25]
+        ;; Workaround for the error "void-variable org-element-citation-prefix-re".
+        ;; Actually, just (require 'org-element) is enough,
+        ;; but for future robustness, we initialize org-mode entirely for now.
+        ;; https://github.com/kaorahi/howm/issues/65
+        (with-temp-buffer (org-mode))  ;; workaround (#65)
+        (require 'org)
+        (org-set-font-lock-defaults)
+        (setq-local font-lock-keywords nil)
+        ;; restore howm's font-lock (#65)
+        (cond ((eq major-mode 'howm-view-summary-mode)
+               (howm-view-summary-mode-body))
+              ((eq major-mode 'howm-view-contents-mode)
+               (howm-view-contents-mode-body)))
+        (font-lock-add-keywords nil org-font-lock-keywords))
+    (font-lock-remove-keywords nil org-font-lock-keywords))
+  (font-lock-flush)
+  (font-lock-ensure))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Bug Report
 
@@ -960,10 +1021,10 @@ When DOTS-STR is non-nil, it is used instead of \"...\"."
 
 (defun howm-set-lang ()
   (set-language-environment "Japanese")
-  (set-default-coding-systems 'euc-jp)
-  (set-buffer-file-coding-system 'euc-jp-unix)
-  (set-terminal-coding-system 'euc-jp)
-  (set-keyboard-coding-system 'euc-jp)
+  (set-default-coding-systems 'utf-8)
+  (set-buffer-file-coding-system 'utf-8-unix)
+  (set-terminal-coding-system 'utf-8)
+  (set-keyboard-coding-system 'utf-8)
   )
 
 (defun howm-compiled-p ()
@@ -996,7 +1057,7 @@ When DOTS-STR is non-nil, it is used instead of \"...\"."
        (format "grep: %s - %s\n"
                (cl-mapcan (lambda (d)
                                  (let ((f (expand-file-name
-                                           howm-view-grep-command d)))
+                                           (howm-grep-command) d)))
                                    (and (file-executable-p f)
                                         (list f))))
                                exec-path)
