@@ -1,6 +1,7 @@
+;;; -*- lexical-binding: nil; -*-
 ;;; howm-mode.el --- Wiki-like note-taking tool
-;;; Copyright (C) 2002, 2003, 2004, 2005-2022
-;;;   HIRAOKA Kazuyuki <khi@users.osdn.me>
+;;; Copyright (C) 2002, 2003, 2004, 2005-2025
+;;;   HIRAOKA Kazuyuki <kakkokakko@gmail.com>
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -78,6 +79,9 @@ If it is a function, it is called to get template string with the argument <n>."
     ("c" howm-create t t)
     ("e" howm-remember t t)
     ("," howm-menu t t)
+    ;; "C-c ; ;" for org
+    ;; https://github.com/kaorahi/howm/issues/38#issuecomment-2640020651
+    (";" howm-menu t t)
     ("." howm-find-today nil t)
     (":" howm-find-yesterday nil t)
     ("A" howm-list-around)
@@ -86,7 +90,7 @@ If it is a function, it is called to get template string with the argument <n>."
     ("i" howm-insert-keyword nil t)
     ("d" howm-insert-date nil t)
     ("T" howm-insert-dtime nil t)
-    ("K" howm-keyword-to-kill-ring t t)
+    ("K" howm-keyword-to-kill-ring nil t)
     ("n" action-lock-goto-next-link)
     ("p" action-lock-goto-previous-link)
     ("Q" howm-kill-all t t)
@@ -110,7 +114,7 @@ It is further registered globally if global-p is non-nil."
 (howm-defvar-risky howm-migemo-client nil
   "Command name of migemo-client.
 Example of cmigemo:
-  (setq howm-migemo-client '((type . cmigemo) (command . \"cmigemo\")))
+  (setq howm-migemo-client \\='((type . cmigemo) (command . \"cmigemo\")))
 Example of migemo-client (obsolete):
   (setq howm-migemo-client \"migemo-client\")
 See also `howm-migemo-client-option`")
@@ -118,9 +122,9 @@ See also `howm-migemo-client-option`")
   "List of option for migemo-client.
 Example of cmigemo:
   (setq howm-migemo-client-option
-        '(\"-q\" \"-d\" \"/usr/share/cmigemo/utf-8/migemo-dict\"))
+        \\='(\"-q\" \"-d\" \"/usr/share/cmigemo/utf-8/migemo-dict\"))
 Example of migemo-client (obsolete):
-  (setq howm-migemo-client-option '(\"-H\" \"::1\")
+  (setq howm-migemo-client-option \\='(\"-H\" \"::1\")
 See also `howm-migemo-client`")
 
 ;;; --- level 2 ---
@@ -128,23 +132,28 @@ See also `howm-migemo-client`")
 ;; Be careful to keep consistency.
 
 (howm-defvar-risky howm-keyword/ref-regexp-format
-  "\\(%s\\)[ \t]*\\([^ \t\r\n].*\\)")
+  "\\(%s\\)[ \t]*\\(%s\\)")
 (howm-defvar-risky howm-keyword-format
   (format "%s %%s" howm-keyword-header)
   "Format for declaration of keyword. See `format'.")
+(howm-defvar-risky howm-keyword-body-regexp "[^ \t\r\n].*")
 (howm-defvar-risky howm-keyword-regexp
-  (format howm-keyword/ref-regexp-format (regexp-quote howm-keyword-header)))
+  (format howm-keyword/ref-regexp-format
+          (regexp-quote howm-keyword-header) howm-keyword-body-regexp))
 (howm-defvar-risky howm-keyword-regexp-hilit-pos 1)
 (howm-defvar-risky howm-keyword-regexp-pos 2)
+(howm-defvar-risky howm-ref-body-regexp howm-keyword-body-regexp)
 (howm-defvar-risky howm-ref-regexp
-  (format howm-keyword/ref-regexp-format (regexp-quote howm-ref-header))
+  (format howm-keyword/ref-regexp-format
+          (regexp-quote howm-ref-header) howm-ref-body-regexp)
   "Regexp for explicit link.")
 (howm-defvar-risky howm-ref-regexp-hilit-pos 0
   "Position of search string in `howm-ref-regexp'")
 (howm-defvar-risky howm-ref-regexp-pos 2
   "Position of search string in `howm-ref-regexp'")
 (howm-defvar-risky howm-wiki-regexp "\\[\\[\\([^]\r\n]+\\)\\]\\]"
-  "Regexp for explicit link.")
+  "Regexp for explicit link. Set this to nil to disable the feature
+  entirely.")
 (howm-defvar-risky howm-wiki-regexp-hilit-pos 1
   "Position of hilight in `howm-wiki-regexp'")
 (howm-defvar-risky howm-wiki-regexp-pos 1
@@ -217,10 +226,11 @@ in `howm-template'. %s is replaced with name of last file. See `format'.")
          ;;           (mapconcat #'howm-action-lock-quote-keyword ks "\\|"))
          ;;          (t
          ;;           (regexp-opt ks (and howm-check-word-break 'word)))))
-         (wiki (howm-action-lock-search howm-wiki-regexp
-                                        howm-wiki-regexp-pos
-                                        howm-wiki-regexp-hilit-pos
-                                        t))
+         (wiki (and howm-wiki-regexp
+                    (list (howm-action-lock-search howm-wiki-regexp
+                                                   howm-wiki-regexp-pos
+                                                   howm-wiki-regexp-hilit-pos
+                                                   t))))
          (explicit (howm-action-lock-search howm-ref-regexp
                                             howm-ref-regexp-pos
                                             howm-ref-regexp-hilit-pos))
@@ -234,7 +244,7 @@ in `howm-template'. %s is replaced with name of last file. See `format'.")
                 ,explicit
                 ,rev
                 ,@(if ks (list implicit) nil)
-                ,wiki
+                ,@wiki
                 ,@(if (howm-menu-p) nil (list date done))
                 ))
          )
@@ -250,18 +260,16 @@ in `howm-template'. %s is replaced with name of last file. See `format'.")
 ;; Definitions
 
 (define-minor-mode howm-mode
-  "With no argument, this command toggles the mode. 
-Non-null prefix argument turns on the mode.
-Null prefix argument turns off the mode.
+  "Toggle Howm mode.
 
 When the mode is enabled, underlines are drawn on texts which match
-to titles of other files. Typing \\[action-lock-magic-return] there,
+to titles of other files. Typing \\<action-lock-mode-map>\\[action-lock-magic-return] there,
 you can jump to the corresponding file.
 
 key	binding
 ---	-------
 \\[action-lock-magic-return]	Follow link
-\\[howm-refresh]	Refresh buffer
+\\<howm-mode-map>\\[howm-refresh]	Refresh buffer
 \\[howm-list-all]	List all files
 \\[howm-list-grep]	Search (grep)
 \\[howm-create]	Create new file
@@ -314,6 +322,13 @@ key	binding
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Main functions
 
+(defun howm-refresh-all-buffers ()
+  (mapc (lambda (buf)
+          (with-current-buffer buf
+            (when howm-mode
+              (howm-refresh))))
+        (buffer-list)))
+
 (defun howm-refresh ()
   (interactive)
   (if (howm-menu-p)
@@ -332,13 +347,18 @@ key	binding
       (howm-mode-add-font-lock)
       (howm-reminder-add-font-lock)
       (cheat-font-lock-fontify)
-      ;; make-local-hook is obsolete for emacs >= 21.1.
-      (howm-funcall-if-defined (make-local-hook 'after-save-hook))
+      (add-hook 'before-save-hook 'howm-before-save t t)
       (add-hook 'after-save-hook 'howm-after-save t t))))
 
+(defun howm-before-save ()
+  ;; Update the keyword list BEFORE saving the note
+  ;; so that the "Wrote ..." message is not overshadowed.
+  ;; https://github.com/kaorahi/howm/pull/32#issuecomment-2607371907
+  (when howm-mode
+    (howm-keyword-add-current-buffer)))
+
 (defun howm-after-save ()
   (when howm-mode
-    (howm-keyword-add-current-buffer)
     (when howm-refresh-after-save
       (howm-initialize-buffer))
     (when (and howm-menu-refresh-after-save
@@ -352,10 +372,14 @@ key	binding
 (defun howm-list-all ()
   (interactive)
   (howm-set-command 'howm-list-all)
-  (howm-normalize-show "" (howm-all-items))
-  ;; for backward compatibility
-  (cond ((howm-list-title-p) t)  ;; already done in howm-normalize-show
-        (howm-list-all-title (howm-list-title-internal))))
+  (let ((all-items (howm-all-items)))
+    (if (null all-items)
+        (when (y-or-n-p "No notes yet. Create?")
+          (howm-create))
+      (howm-normalize-show "" all-items)
+      ;; for backward compatibility
+      (cond ((howm-list-title-p) t)  ;; already done in howm-normalize-show
+            (howm-list-all-title (howm-list-title-internal))))))
 
 (defun howm-all-items ()
   "Returns list of all items in the first search path."
@@ -367,12 +391,17 @@ key	binding
   (let* ((d (or days howm-list-recent-days))
          (now (current-time))
          (from (howm-days-before now d))
-         (item-list (howm-folder-items howm-directory t)))
-    (howm-normalize-show "" (howm-filter-items-by-mtime item-list from now))
-    ;; clean me [2003-11-30]
-    (cond ((howm-list-title-p) t)  ;; already done in howm-normalize-show
-          (howm-list-recent-title (howm-list-title-internal))
-          ((not days) (howm-view-summary-to-contents)))))
+         (item-list (howm-recent-items-filter
+                     (howm-folder-items howm-directory t)))
+         (recent-items (howm-filter-items-by-mtime item-list from now)))
+    (if (null recent-items)
+        (when (y-or-n-p "No recent notes. Create?")
+          (howm-create))
+      (howm-normalize-show "" recent-items)
+      ;; clean me [2003-11-30]
+      (cond ((howm-list-title-p) t)  ;; already done in howm-normalize-show
+            (howm-list-recent-title (howm-list-title-internal))
+            ((not days) (howm-view-summary-to-contents))))))
 
 ;; clean me: direct access to howm-view-* is undesirable.
 
@@ -402,7 +431,7 @@ key	binding
   (let ((b (current-buffer)))
     (howm-list-title-put-previous)
     (howm-view-list-title (howm-list-title-regexp))
-    ;;       (howm-view-filter-by-contents (howm-list-title-regexp))
+    ;;       (howm-view-grep-in-contents (howm-list-title-regexp))
     (let ((c (current-buffer)))
       (when (not (eq b c))
         (set-buffer b)
@@ -457,7 +486,8 @@ key	binding
         (when filter
           (setq items (funcall filter items)))
         (howm-normalize-show name items (or emacs-regexp regexp) nil nil kw)
-        (howm-record-view-window-configuration)))))
+        (howm-record-view-window-configuration)
+        items))))
 
 (defun howm-iigrep (completion-p action)
   (howm-with-iigrep (howm-iigrep-command-for-pattern completion-p)
@@ -889,11 +919,12 @@ We need entire-match in order to
     (cond ((howm-buffer-empty-p) nil)
           ((and here howm-create-here-just) (beginning-of-line))
           (t (howm-create-newline)))
-    (let ((p (point))
-          (insert-f (lambda (switch)
-                      (howm-insert-template (if switch title "")
-                                            b which-template (not switch))))
-          (use-file (not not-use-file)))
+    (let* ((p (point))
+           (template-string (howm-template-string which-template b))
+           (insert-f (lambda (switch)
+                       (howm-insert-template-string template-string (if switch title "")
+                                                    b (not switch))))
+           (use-file (not not-use-file)))
       ;; second candidate which appears when undo is called
       (let ((end (funcall insert-f not-use-file)))
         (save-excursion
@@ -924,10 +955,16 @@ We need entire-match in order to
 
 (defun howm-insert-template (title &optional
                                    previous-buffer which-template not-use-file)
+  (let ((template-string (howm-template-string which-template previous-buffer)))
+    (howm-insert-template-string template-string title
+                                 previous-buffer not-use-file)))
+
+(defun howm-insert-template-string (template-string title &optional
+                                                    previous-buffer not-use-file)
   (let* ((beg (point))
          (f (buffer-file-name previous-buffer))
          (af (and f (howm-abbreviate-file-name f))))
-    (insert (howm-template-string which-template previous-buffer))
+    (insert template-string)
     (let* ((date (format-time-string howm-template-date-format))
            (use-file (not not-use-file))
            (file (cond ((not use-file) "")
@@ -980,17 +1017,22 @@ is necessary.")
          (s (buffer-substring-no-properties (car r) (cadr r))))
     (howm-create-file)
     (howm-set-mode)
-    (insert "\n" s)))
+    (if (howm-buffer-empty-p)
+        (insert s)
+      (howm-create-newline)
+      (insert s)
+      (when howm-prepend
+        (save-excursion (insert "\n\n"))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Keyword
 
-(defun howm-completing-read-keyword ()
+(defun howm-completing-read-keyword (&rest opts)
   (message "Scanning...")
   (let* ((kl (howm-keyword-list))
          (table (mapcar #'list kl))
          (completion-ignore-case howm-keyword-case-fold-search))
-    (completing-read "Keyword: " table)))
+    (apply #'completing-read "Keyword: " table opts)))
 
 (defun howm-insert-keyword ()
   (interactive)
@@ -1052,7 +1094,53 @@ is necessary.")
                 (insert k "\n")))
             keyword-list)
       (when (buffer-file-name)
-        (howm-basic-save-buffer)))))
+        (howm-basic-save-buffer))))
+  (when (called-interactively-p 'interactive)
+    (howm-refresh-all-buffers)))
+
+(defvar howm-keyword-remove-warn-p t)
+
+(defun howm-keyword-remove (keyword)
+  "Remove KEYWORD from the keyword list for come-from links."
+  (interactive (list (howm-completing-read-keyword nil t)))
+  (when howm-keyword-remove-warn-p
+    (let ((prompt (format "Safe to back up \"%s\" yourself before truncating. Truncate now? "
+                          (howm-keyword-file))))
+      (if (yes-or-no-p prompt)
+          (setq howm-keyword-remove-warn-p nil)
+        (error "Abort."))))
+  (unless (string= keyword "")
+    (howm-keyword-remove-sub keyword)
+    (howm-refresh-all-buffers)
+    (let ((prompt (format "Removed \"%s\"; but you may need to edit existing notes. Search for it?"
+                          keyword)))
+      (when (y-or-n-p prompt)
+        (howm-search keyword t)))))
+
+(defun howm-keyword-remove-sub (keyword)
+  (with-current-buffer (howm-keyword-buffer)
+    (save-excursion
+      (save-restriction
+        (widen)
+        (let ((case-fold-search nil))
+          ;; It's inefficient to scan the buffer multiple times, but
+          ;; since this isn't a frequently used feature, the code is
+          ;; kept simple for now. [2025-06-27]
+          (let* ((q (regexp-quote keyword))
+                 (regs (mapcar (lambda (pat) (format pat q))
+                               '("^%s$" "^%s\t" "\t%s$" "\t%s\t")))
+                 (replace-pattern
+                  (lambda (n to)
+                    (goto-char (point-min))
+                    (while (re-search-forward (nth n regs) nil t)
+                      (replace-match to nil nil)))))
+            (goto-char (point-min))
+            (flush-lines (nth 0 regs))
+            (funcall replace-pattern 1 "")
+            (funcall replace-pattern 2 "")
+            (funcall replace-pattern 3 "\t")
+            (when (buffer-file-name)
+              (howm-basic-save-buffer))))))))
 
 (defun howm-keyword-new-p (str)
   (save-excursion
@@ -1238,8 +1326,7 @@ KEYWORD itself is always at the head of the returneded list.
 (defun howm-keyword-add-current-buffer ()
   (save-excursion
     (goto-char (point-min))
-    (let ((m (current-message))
-          (keyword-list nil))
+    (let ((keyword-list nil))
       (while (re-search-forward howm-keyword-regexp nil t)
         (let ((key-str (if howm-keyword-list-alias-sep
                            (mapconcat #'identity
@@ -1247,8 +1334,7 @@ KEYWORD itself is always at the head of the returneded list.
                                       howm-keyword-list-alias-sep)
                          (match-string-no-properties howm-keyword-regexp-pos))))
           (setq keyword-list (cons key-str keyword-list))))
-      (howm-keyword-add keyword-list)
-      (message "%s" m))))
+      (howm-keyword-add keyword-list))))
 (defun howm-keyword-add-items (items)
   (let ((files (mapcar #'howm-view-item-filename items)))
     (with-temp-buffer
