@@ -1,6 +1,7 @@
+;;; -*- lexical-binding: nil; -*-
 ;;; howm-reminder.el --- Wiki-like note-taking tool
-;;; Copyright (C) 2002, 2003, 2004, 2005-2021
-;;;   HIRAOKA Kazuyuki <khi@users.osdn.me>
+;;; Copyright (C) 2002, 2003, 2004, 2005-2025
+;;;   HIRAOKA Kazuyuki <kakkokakko@gmail.com>
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -63,11 +64,6 @@ schedules outside the range in %reminder in the menu.")
 
 ;; Fix me: redundant (howm-date-* & howm-reminder-*)
 
-;; (defun howm-reminder-regexp-grep (types)
-;;   (howm-inhibit-warning-in-compilation))
-;; (defun howm-reminder-regexp (types)
-;;   (howm-inhibit-warning-in-compilation))
-
 (if howm-reminder-old-format
     (progn ;; old format
       (defvar howm-reminder-regexp-grep-format
@@ -275,10 +271,14 @@ This value is passed to `format-time-string', and the result must be a regexp."
     :group 'howm-faces))
 
 (defun howm-reminder-today-font-lock-keywords ()
-  (let ((today    (howm-reminder-today 0 howm-highlight-date-regexp-format))
-        (tomorrow (howm-reminder-today 1 howm-highlight-date-regexp-format)))
+  (let ((today    (howm-reminder-today-matcher 0))
+        (tomorrow (howm-reminder-today-matcher 1)))
     `((,today (0 howm-reminder-today-face prepend))
       (,tomorrow (0 howm-reminder-tomorrow-face prepend)))))
+(defun howm-reminder-today-matcher (n)
+  (let* ((n1 (+ n (howm-todo-priority-get-late-offset)))
+         (r (howm-reminder-today n1 howm-highlight-date-regexp-format)))
+    `(lambda (bound) (re-search-forward ,r bound t))))
 
 (defun howm-reminder-add-font-lock ()
   (cheat-font-lock-append-keywords (howm-reminder-add-font-lock-internal)))
@@ -384,10 +384,116 @@ This value is passed to `format-time-string', and the result must be a regexp."
   (interactive)
   (howm-list-todo-sub #'howm-todo-sleeping-p))
 
+(defun howm-simulate-todo ()
+  "Start a time machine simulation of the todo list.
+See `howm-simulate-todo-mode' for details. See also
+`howm-simulate-todo-next-date', `howm-simulate-todo-previous-date',
+and `howm-simulate-todo-reset'."
+  (interactive)
+  (howm-simulate-todo-reset (howm-simulate-todo-setup-items nil))
+  (goto-char (point-min))
+  (let ((keys (mapcar (lambda (f)
+                        (let ((k (car-safe (where-is-internal f))))
+                          (if k (key-description k) (format "M-x %s" f))))
+                      '(howm-simulate-todo-previous-date
+                        howm-simulate-todo-next-date
+                        howm-simulate-todo-reset))))
+    (apply #'message
+           "Time machine simulation: Type %s and %s to navigate, %s to reset."
+           keys))
+  (sit-for 5))
+(define-derived-mode howm-simulate-todo-mode howm-view-summary-mode "HowmT"
+  "Major mode for a time machine simulation of the todo list in howm.
+(Tips) Type C-u 30 \\<howm-simulate-todo-mode-map>\\[howm-simulate-todo-next-date] to simulate 30 days later.
+
+key	binding
+---	-------
+\\[howm-simulate-todo-next-date]	Simulate the next date
+\\[howm-simulate-todo-previous-date]	Simulate the previous date
+\\[howm-simulate-todo-reset]	Reset to today
+"
+  ;; major mode just for additional key bindings
+  (howm-view-summary-mode-body)
+  (action-lock-mode -1))
+(let ((m howm-simulate-todo-mode-map))
+  (mapc (lambda (args) (apply #'define-key m args))
+        '((">" howm-simulate-todo-next-date)
+          ("<" howm-simulate-todo-previous-date)
+          ("=" howm-simulate-todo-reset))))
+(defun howm-simulate-todo-setup-items (items)
+  (let ((howm-todo-separators nil))
+    (howm-list-todo-sub-setup-items items)))
+(defun howm-simulate-todo-next-date (n &optional orig-items)
+  "Simulate the todo list as if today were N days after the current
+\\='today\\='.  If N is nil, reset the simulation to today\\='s date.
+If ORIG-ITEMS is nil, use the item list of the current buffer.  For
+example, to simulate 30 days later, call this function with the prefix
+C-u 30."
+  (interactive "p")
+  (howm-with-simulated-date
+    (let* ((ti (howm-increment-simulated-date n))
+           (msg (format-time-string " %Y-%m-%d (%a) <=>" ti))
+           (items (howm-simulate-todo-setup-items (or orig-items
+                                                      (howm-view-item-list)))))
+      (when items
+        (let* ((r (howm-reminder-regexp howm-todo-types))
+               (summarizer (howm-reminder-summarizer r t))
+               (f (lambda (item)
+                    (funcall summarizer 'dummy 'dummy (howm-item-summary item)))))
+          (mapc (lambda (item) (howm-item-set-summary item (funcall f item)))
+                items))
+        (howm-save-vertical-position
+          (howm-view-summary-rebuild items)
+          (howm-list-reminder-final-setup "{sim}" items))
+        (howm-list-todo-insert-priority items)
+        (howm-simulate-todo-mode)
+        (setq howm-view-mode-line-text
+              (propertize msg 'face howm-simulate-todo-mode-line-face))))))
+(defun howm-simulate-todo-previous-date (n &optional orig-items)
+  "Simulate the todo list as if today were N days before the current \\='today\\='.
+If N is nil, reset the simulation to today\\='s date.
+If ORIG-ITEMS is nil, use the item list of the current buffer.
+For example, to simulate 30 days ago, call this function with the prefix C-u 30."
+  (interactive "p")
+  (howm-simulate-todo-next-date (and n (- n)) orig-items))
+(defun howm-simulate-todo-reset (&optional orig-items)
+  "Reset the simulated todo list to today's date.
+If ORIG-ITEMS is nil, use the item list of the current buffer."
+  (interactive)
+  (howm-simulate-todo-next-date nil orig-items))
+(defmacro howm-save-vertical-position (&rest body)
+  (declare (indent 0))
+  (let ((gw (cl-gensym))
+        (gc (cl-gensym)))
+    `(let ((,gw (line-number-at-pos (window-start) t))
+           (,gc (line-number-at-pos nil t)))
+       (prog1
+           (progn ,@body)
+         (howm-goto-line ,gw)
+         (recenter 0)
+         (howm-goto-line ,gc)))))
+(defun howm-list-todo-insert-priority (item-list)
+  (save-excursion
+    (let ((buffer-read-only nil))
+      (buffer-disable-undo)
+      (goto-char (point-min))
+      (mapc (lambda (item)
+              (when (search-forward howm-view-summary-sep (line-end-position) t)
+                  (insert (format "%8.1f" (howm-todo-priority item))))
+              (forward-line 1))
+            item-list))))
+
 (defun howm-list-todo-sub (&optional pred)
+  (let ((items (howm-list-todo-sub-setup-items nil pred)))
+    (when items
+      (howm-list-reminder-final-setup howm-list-todo-name items))))
+
+(defun howm-list-todo-sub-setup-items (given-items &optional pred)
+  (when (null given-items)
+    (setq given-items (howm-list-reminder-internal howm-todo-types)))
   (howm-with-need
     (howm-with-schedule-summary-format
-      (let ((items (need (howm-list-reminder-internal howm-todo-types))))
+      (let ((items (need given-items)))
         (when pred
           (setq items
                 (need (cl-remove-if-not pred items))))
@@ -396,12 +502,14 @@ This value is passed to `format-time-string', and the result must be a regexp."
           (setq items
                 (howm-todo-insert-separators items
                                              howm-todo-separators)))
-      (howm-list-reminder-final-setup howm-list-todo-name items)))))
+        items))))
 
-(defun howm-todo-menu (n limit-priority separators)
+(defun howm-todo-menu (n limit-priority separators &optional must-show-priority)
   "Find top N todo items, or all todo items if N is nil.
 Returned value is a sorted list of items (see `howm-make-item').
 Items whose priority is worse than LIMIT-PRIORITY are eliminated.
+If MUST-SHOW-PRIORITY is given, items whose priority is equal to
+or better than it are included, even if they are out of the top N.
 Separator strings are inserted to the returned list according to
 the rule given as SEPARATORS.
 See docstring of the variable `howm-menu-reminder-separators' for details."
@@ -409,13 +517,22 @@ See docstring of the variable `howm-menu-reminder-separators' for details."
                                       (< (howm-todo-priority item)
                                          limit-priority))
                                     (howm-reminder-search howm-todo-menu-types)))
-         (sorted (howm-todo-sort-items cutted)))
-    (howm-todo-insert-separators (if n (howm-first-n sorted n) sorted)
-                                 separators t)))
+         (sorted (howm-todo-sort-items cutted))
+         (count 0)
+         (filtered (cl-loop for item in sorted
+                            if (or (< count n)
+                                   (and must-show-priority
+                                        (>= (howm-todo-priority item)
+                                            must-show-priority)))
+                            collect (prog1 item
+                                      (unless (equal (cadr (howm-todo-parse item)) "@")
+                                        (cl-incf count))))))
+    (howm-todo-insert-separators filtered separators t)))
 
 (defun howm-reminder-menu (n limit-priority separators)
   (howm-with-reminder-setting
-    (howm-todo-menu n limit-priority separators)))
+    (howm-todo-menu n limit-priority separators
+                    (- howm-todo-priority-schedule-top howm-huge-))))
 
 (defun howm-todo-insert-separators (item-list separators
                                               &optional relative-date-p)
@@ -440,6 +557,27 @@ See docstring of the variable `howm-menu-reminder-separators' for details."
 (defun howm-todo-sort-by-priority ()
   (howm-view-sort-doit #'howm-todo-sort-items))
 
+(defvar howm-todo-priority-late-offset 0
+  "For internal use")
+(make-variable-buffer-local 'howm-todo-priority-late-offset)
+(defvar howm-todo-priority-late-offset-enabled nil
+  "For internal use")
+(make-variable-buffer-local 'howm-todo-priority-late-offset-enabled)
+(defun howm-todo-priority-get-late-offset ()
+  (if howm-todo-priority-late-offset-enabled
+      howm-todo-priority-late-offset
+    0))
+(defmacro howm-with-simulated-date (&rest body)
+  (declare (indent 0))
+  `(let ((howm-todo-priority-late-offset-enabled t))
+     ,@body))
+(defun howm-increment-simulated-date (increment)
+  (setq howm-todo-priority-late-offset
+        (if increment
+            (+ howm-todo-priority-late-offset increment)
+          0))
+  (howm-days-after (current-time) howm-todo-priority-late-offset))
+
 ;; Clean me.
 (defun howm-reminder-parse (item)
   (howm-todo-parse-string (howm-view-item-summary item)))
@@ -464,7 +602,7 @@ Example: (howm-todo-parse-string \"abcde [2004-11-04]@ hogehoge\")
           (description (substring str (match-end 0))))
       (let* ((day (howm-encode-day d m y))
              (today (howm-encode-day))
-             (late (- today day))
+             (late (+ (- today day) (howm-todo-priority-get-late-offset)))
              (type (substring (or ty "-") 0 1)) ;; "-" for old format
              (lazy (cond ((string= type " ") nil)
                          ((null lz) nil)
@@ -880,8 +1018,8 @@ or TODO is t."
   "Modify variable VAR whose value is \"[...]\".
 Example:
  (setq foo \"[abc]\")
- (howm-modify-reminder-types 'foo \"d\" t)  foo ==> \"[abcd]\"
- (howm-modify-reminder-types 'foo \"b\" nil)  foo ==> \"[acd]\"
+ (howm-modify-reminder-types \\='foo \"d\" t)  foo ==> \"[abcd]\"
+ (howm-modify-reminder-types \\='foo \"b\" nil)  foo ==> \"[acd]\"
 "
   (let ((val (symbol-value var)))
     (when (not (string-match "^\\[\\(.*\\)\\]$" val))
