@@ -1,6 +1,7 @@
+;;; -*- lexical-binding: nil; -*-
 ;;; howm-vars.el --- Wiki-like note-taking tool
-;;; Copyright (C) 2005-2021
-;;;   HIRAOKA Kazuyuki <khi@users.osdn.me>
+;;; Copyright (C) 2005-2025
+;;;   HIRAOKA Kazuyuki <kakkokakko@gmail.com>
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -77,7 +78,7 @@
 ;;      (howm-dont-warn-free-variable ,var)))
 (defmacro howm-dont-warn-free-variable (var)
   "No effect except for inhibition of warning in byte-compilation.
-Without this trick, compiler says 'reference to free variable' even when
+Without this trick, compiler says \"reference to free variable\" even when
 we have checked availability like (if (boundp xxx) ...)."
   `(when (boundp (quote ,var))
      (defvar ,var nil)))
@@ -87,11 +88,11 @@ we have checked availability like (if (boundp xxx) ...)."
 Otherwise, execute expressions in NOT-DEFINED.
 This is cheat to avoid warning while byte-compilation.
 Byte-compiler says \"not known to be defined\" even for codes like
-  (if (fboundp 'foo) (foo bar)).
+  (if (fboundp \\='foo) (foo bar)).
 
-\(macroexpand '(howm-funcall-if-defined (migemo-get-pattern roma) nil))
-==> (if (fboundp 'migemo-get-pattern)
-        (let ((howm-funcall-if-defined-f 'migemo-get-pattern))
+\(macroexpand \\='(howm-funcall-if-defined (migemo-get-pattern roma) nil))
+==> (if (fboundp \\='migemo-get-pattern)
+        (let ((howm-funcall-if-defined-f \\='migemo-get-pattern))
           (funcall howm-funcall-if-defined-f roma))
       nil)
 "
@@ -125,7 +126,7 @@ Byte-compiler says \"not known to be defined\" even for codes like
 (defvar howm-compatible-to-ver1dot3 nil
   "If non-nil, compatible values to howm-1.3.* are used
 as default of some variables; put (setq howm-compatible-to-ver1dot3 t)
-*before* (require 'howm) if you like.")
+*before* (require \\='howm) if you like.")
 
 (defgroup howm-compatibility nil
   "Compatibility to howm-1.3.*."
@@ -277,15 +278,14 @@ A file is excluded iff this regexp matches with all the relative paths."
   :group 'howm-efficiency
   :group 'howm-menu)
 
-(defcustom howm-menu-name-format "*howmM:%s*"
-  "*Name format of menu buffer."
-  :type '(radio (const :tag "Never show in normal buffer list" " *howmM:%s*")
-                string)
-  :group 'howm-menu)
+(defvar howm-menu-name-format "*howmM:%s*"
+  "*Name format of menu buffer. White spaces at the beginning are not
+allowed.")
 
 (defcustom howm-menu-footer nil
-  "Footer string for each menu. Nil means no footer."
-  :type '(radio (const :tag "Off" nil)
+  "Footer string for each menu."
+  :type '(radio (const :tag "Default (function `howm-menu-footer')" nil)
+                (const :tag "No footer" "")
                 string)
   :group 'howm-menu)
 
@@ -575,6 +575,15 @@ if `howm-list-normalizer' is non-nil."
   :type 'boolean
   :group 'howm-sort)
 
+(defcustom howm-recent-excluded-files-regexp nil
+  "Regexp matching names of files to exclude from recents.
+This affects the files shown in the recents section of the
+Howm-menu, as well as the files listed by `howm-list-recent'."
+  :type '(radio (const :tag "Default" nil)
+                regexp)
+  :group 'howm-menu-reminder
+  :group 'howm-files)
+
 ;;
 ;; Title
 ;;
@@ -608,7 +617,7 @@ if `howm-list-normalizer' is non-nil."
     howm-action-lock-date-search
     )
   "List of commands in which titles are listed instead of matched lines.
-T means 'always'.
+T means \"always\".
 If it is a function, the evaluated value is used instead of itself."
   :type `(radio (const :tag "Always" t)
                 (const :tag "Never" nil)
@@ -748,19 +757,29 @@ If it is nil, alias of come-from keyword is disabled."
   :group 'howm-efficiency
   :group 'howm-search)
 
-(howm-defcustom-risky howm-view-use-grep nil
-  "*If non-nil, use external grep command for search.
+(howm-defcustom-risky howm-view-use-grep
+  ;; disabled for windows by default
+  ;; https://github.com/kaorahi/howm/issues/6
+  (cdr (assoc system-type
+              '((gnu/linux . auto)
+                (darwin . auto)
+                (windows-nt . nil))))
+  "*If this is t, use external grep command for search.
 Performance must be improved greatly if you set this.
+If this is the symbol `auto', use external grep command unless it
+causes an error.
 When the value is elisp function, it is used instead of `howm-fake-grep'."
   :type '(radio (const :tag "On" t)
                 (const :tag "Off" nil)
+                (const :tag "Auto" auto)
                 function)
   :group 'howm-grep)
 
 ;; These variables should be renamed: howm-view-xxx ==> howm-xxx.
-(howm-defcustom-risky howm-view-grep-command "grep"
-  "*Command name for grep."
-  :type 'string
+(howm-defcustom-risky howm-view-grep-command nil
+  "*Command name for grep. If it is nil, `grep-program' is used."
+  :type '(radio (const :tag "Default (grep-program)" nil)
+                string)
   :group 'howm-grep)
 (howm-defvar-risky howm-view-fgrep-command nil
   "*Command name for fgrep.
@@ -769,8 +788,8 @@ This variable is obsolete and may be removed in future.")
   ;; "labels" causes a trouble in git-head emacs (d5e3922) [2015-01-31]
   (let* ((ed (lambda (d) (concat "--exclude-dir=" d)))
          (has-ed (condition-case nil
-                     (eq 0 (call-process howm-view-grep-command nil nil nil
-                                         (apply ed "/") "--version"))
+                     (eq 0 (call-process (howm-grep-command) nil nil nil
+                                         (funcall ed "/") "--version"))
                      (error nil)))
          (opts (cons "-Hnr" (and has-ed (mapcar ed howm-excluded-dirs)))))
     (mapconcat #'identity opts " ")))
@@ -818,8 +837,8 @@ If the value is a cons pair, its car and cdr are used for read and write,
 respectively.
 
 Example:
- (setq howm-process-coding-system 'euc-japan-unix)
- (setq howm-process-coding-system '(utf-8-unix . sjis-unix))"
+ (setq howm-process-coding-system \\='euc-japan-unix)
+ (setq howm-process-coding-system \\='(utf-8-unix . sjis-unix))"
   :type '(radio (const :tag "Off" nil)
                 coding-system
                 (cons coding-system coding-system))
@@ -832,6 +851,35 @@ so that highlighting works correctly."
     :type 'boolean
     :group 'howm-grep))
 
+(defgroup howm-iigrep nil
+  "Show search results on the fly before hitting RET."
+  :group 'howm-grep)
+
+(defcustom howm-iigrep-preview-items 20
+  "Show howm search results on the fly before hitting RET when the hit
+counts are within this number."
+  :type 'integer
+  :group 'howm-efficiency
+  :group 'howm-iigrep)
+
+(defcustom howm-iigrep-show-what 'counts
+  "Show search results on the fly before hitting RET."
+  :type '(radio (const :tag "Off" nil)
+                (const :tag "Counts" counts)
+                (const :tag "Contents" contents)
+                (const :tag "Full" full))
+  :group 'howm-efficiency
+  :group 'howm-iigrep)
+
+(defcustom howm-iigrep-migemo-show-what 'inherit
+  "Show migemo search results on the fly before hitting RET."
+  :type '(radio (const :tag "Inherit howm-iigrep-show-what" inherit)
+                (const :tag "Counts" counts)
+                (const :tag "Contents" contents)
+                (const :tag "Full" full))
+  :group 'howm-efficiency
+  :group 'howm-iigrep)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Misc
 
@@ -847,6 +895,13 @@ so that highlighting works correctly."
   :type 'number
   :group 'howm-misc)
 
+(defcustom howm-remember-insertion-format "%s\n"
+  "*Format for insertion of the input text in `howm-remember'."
+  :type '(radio (const :tag "Blank line after the text." "%s\n")
+                (const :tag "No Blank line after the text." "%s")
+                string)
+  :group 'howm-misc)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Create
 
@@ -916,7 +971,7 @@ and `howm-viewer-type' accept functions instead of format strings.
 
 Example:
   (setq howm-view-external-viewer-assoc
-        '(
+        \\='(
           (\"[.]\\(jpg\\|gif\\|png\\)$\" . \"display %s\")
           (\"[.]dvi$\" . \"xdvi %s\")
          ))
@@ -934,7 +989,15 @@ cause conflicts."
   :type 'boolean
   :group 'howm-viewer)
 
-(defcustom howm-view-open-by-myself '("text/.*" "application/emacs-lisp")
+(defcustom howm-view-open-by-myself
+  (list "text/.*"
+        "application/emacs-lisp"
+        ;; Workaround for *.org in Gentoo Linux [2025-04-12]
+        ;; /etc/mime.types says that "*.org" is
+        ;; "application/vnd.lotus-organizer", and
+        ;; /etc/mailcap associates "application/*" with "less".
+        ;; https://github.com/kaorahi/howm/issues/64
+        (regexp-quote "application/vnd.lotus-organizer"))
   "List of regular expressions for mime types which should be opened normally."
   :type '(repeat regexp)
   :group 'howm-viewer)
@@ -948,7 +1011,7 @@ cause conflicts."
 
 (defcustom howm-auto-narrow t
   "List of commands after which the function `howm-auto-narrow' can work.
-If the value is t, it means 'always'."
+If the value is t, it means \"always\"."
   :type `(radio (const :tag "Never" nil)
                 (const :tag "Always" t)
                 ,howm-custom-command-list)
@@ -991,6 +1054,27 @@ Note: `howm-menu-font-lock-rules' overrides this variable."
   :type 'boolean
   :group 'howm-faces)
 
+(defcustom howm-follow-theme nil
+  "*If non-nil, use colors that fit the currently active theme.
+This variable should be set before loading howm.  When you modify its
+value, you need to restart Emacs to apply it."
+  :type 'boolean
+  :group 'howm-faces)
+
+(defcustom howm-auto-theme-custom-entries nil
+  "*Entries to `custom-set-faces' that override the defaults for
+  `howm-follow-theme'.
+
+;; example
+(setq howm-follow-theme t)
+(setq howm-auto-theme-custom-entries
+      \\='(
+        (howm-view-hilit-face ((t :inherit isearch)))
+        (howm-view-name-face ((t :inherit org-document-info)))
+        ))"
+  :type '(repeat sexp)
+  :group 'howm-faces)
+
 (defface howm-view-hilit-face
   '((((class color)) (:foreground "red"))
     (t ()))
@@ -1092,6 +1176,12 @@ Note: `howm-menu-font-lock-rules' overrides this variable."
   "*Face for key binding in menu."
   :group 'howm-faces)
 
+(defface howm-simulate-todo-mode-line-face
+  ;; very vivid colors for caution
+  '((t (:foreground "white" :background "red" :weight bold)))
+  "*Face for mode line of todo simulation."
+  :group 'howm-faces)
+
 (defvar howm-view-hilit-face 'howm-view-hilit-face
   "*Face for matched word.")
 (defvar howm-view-name-face  'howm-view-name-face
@@ -1129,6 +1219,9 @@ Note: `howm-menu-font-lock-rules' overrides this variable."
 (defvar howm-menu-key-face  'howm-menu-key-face
   "*Face for key binding in menu.")
 
+(defvar howm-simulate-todo-mode-line-face 'howm-simulate-todo-mode-line-face
+  "*Face for mode line of todo simulation.")
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Hook
 
