@@ -1,6 +1,7 @@
+;;; -*- lexical-binding: nil; -*-
 ;;; howm-view.el --- Wiki-like note-taking tool
-;;; Copyright (C) 2002, 2003, 2004, 2005-2021
-;;;   HIRAOKA Kazuyuki <khi@users.osdn.me>
+;;; Copyright (C) 2002, 2003, 2004, 2005-2025
+;;;   HIRAOKA Kazuyuki <kakkokakko@gmail.com>
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -66,14 +67,18 @@
 (howm-defvar-risky howm-view-filter-methods
   '(("name" . howm-view-filter-by-name)
     ("summary" . howm-view-filter-by-summary)
+    ("keyword" . howm-view-filter-by-keyword-in-summary)
+    ("Keyword-in-contents" . howm-view-actually-filter-by-keyword-in-contents)
+    ("Grep-keyword" . howm-view-grep-keyword-in-contents)
     ("mtime" . howm-view-filter-by-mtime)
 ;     ("ctime" . howm-view-filter-by-ctime) ;; needless
     ("date" . howm-view-filter-by-date)
     ("reminder" . howm-view-filter-by-reminder)
-    ("contents" . howm-view-filter-by-contents)
+    ("contents" . howm-view-actually-filter-by-contents)
+    ("grep" . howm-view-grep-in-contents)
     ("Region" . howm-view-filter-by-region)
     ("Around" . howm-view-filter-by-around)
-;     ("uniq" . howm-view-filter-uniq))
+    ("uniq" . howm-view-filter-uniq)
   ))
 
 ;; referred only when howm-view-use-grep is nil
@@ -102,8 +107,10 @@
   howm-view-summary-name)
 (defun riffle-contents-name-format:howm ()
   howm-view-contents-name)
+(defvar *howm-show-item-filename* t)
 (defun riffle-post-update:howm (item)
-  (howm-message-nolog "View: %s" (howm-view-item-filename item)))
+  (when *howm-show-item-filename*
+    (howm-message-nolog "View: %s" (howm-view-item-filename item))))
 
 ;;; aliases
 
@@ -140,8 +147,6 @@
 
 ;;; variables
 
-(defvar howm-view-font-lock-silent t
-  "Inhibit font-lock-verbose if non-nil.")
 (howm-defvar-risky howm-view-summary-font-lock-keywords
   `((,(concat "\\(^[^ \t\r\n].*?\\)" (regexp-quote howm-view-summary-sep))
      1 howm-view-name-face)
@@ -153,17 +158,18 @@
 This is a shameful global variable and should be clearned in future.")
 (howm-defvar-risky howm-view-font-lock-keywords nil
   "For internal use.")
-(defvar howm-view-font-lock-first-time t
-  "For internal use.")
 (make-variable-buffer-local 'howm-view-font-lock-keywords)
-(make-variable-buffer-local 'howm-view-font-lock-first-time)
+
+(defvar howm-view-mode-line-text ""
+  "For internal use.")
+(make-variable-buffer-local 'howm-view-mode-line-text)
 
 ;;; modes
 
 (riffle-define-derived-mode howm-view-summary-mode riffle-summary-mode "HowmS"
   "memo viewer (summary mode)
 key	binding
----	-------
+---	-------\\<howm-view-summary-mode-map>
 \\[howm-view-summary-open]	Open file
 \\[next-line]	Next item
 \\[previous-line]	Previous item
@@ -183,21 +189,24 @@ key	binding
 \\[howm-list-toggle-title]	Show/Hide Title
 
 \\[howm-view-filter]	Filter (by date, contents, etc.)
-\\[howm-view-filter-by-contents]	Search (= filter by contents)
+\\[howm-view-actually-filter-by-contents]	Filter by contents
+\\[howm-view-actually-filter-by-keyword-in-contents]	Filter by keyword in contents
+\\[howm-view-grep-in-contents]	Search within results
 \\[howm-view-sort]	Sort (by date, summary line, etc.)
 \\[howm-view-sort-reverse]	Reverse order
 \\[howm-view-dired]	Invoke Dired-X
 \\[describe-mode]	This help
-\\[riffle-kill-buffer]	Quit
+\\[howm-view-kill-buffer]	Quit
 "
+  (howm-view-summary-mode-body))
+
+(defun howm-view-summary-mode-body ()
   (make-local-variable 'font-lock-keywords)
-  (cheat-font-lock-mode howm-view-font-lock-silent)
-  (when howm-view-font-lock-first-time
-    (setq howm-view-font-lock-first-time nil)
-    (cheat-font-lock-merge-keywords howm-user-font-lock-keywords
-                                    howm-view-summary-font-lock-keywords
-                                    ;; dirty! Clean dependency between files.
-                                    (howm-reminder-today-font-lock-keywords)))
+  (cheat-font-lock-mode)
+  (cheat-font-lock-merge-keywords howm-user-font-lock-keywords
+                                  howm-view-summary-font-lock-keywords
+                                  ;; dirty! Clean dependency between files.
+                                  (howm-reminder-today-font-lock-keywords))
   (when *howm-view-font-lock-keywords*
     (setq howm-view-font-lock-keywords *howm-view-font-lock-keywords*))
   (when howm-view-font-lock-keywords
@@ -207,6 +216,10 @@ key	binding
   ;; font-lock-set-defaults removes these local variables after 2008-02-24
   (set (make-local-variable 'font-lock-keywords-only) t)
   (set (make-local-variable 'font-lock-keywords-case-fold-search) t)
+  (setq howm-view-mode-line-text "")
+  (setq mode-line-format
+        (cons '(:eval howm-view-mode-line-text)
+              (default-value 'mode-line-format)))
   ;;     (setq font-lock-keywords-case-fold-search
   ;;           howm-view-grep-ignore-case-option)
   (cheat-font-lock-fontify)
@@ -215,7 +228,7 @@ key	binding
 (riffle-define-derived-mode howm-view-contents-mode riffle-contents-mode "HowmC"
   "memo viewer (contents mode)
 key	binding
----	-------
+---	-------\\<howm-view-contents-mode-map>
 \\[howm-view-contents-open]	Open file
 \\[next-line]	Next line
 \\[previous-line]	Previous line
@@ -228,16 +241,21 @@ key	binding
 \\[riffle-contents-goto-previous-item]	Previous item
 
 \\[howm-view-filter]	Filter (by date, contents, etc.)
-\\[howm-view-filter-by-contents]	Search (= filter by contents)
+\\[howm-view-actually-filter-by-contents]	Filter by contents
+\\[howm-view-actually-filter-by-keyword-in-contents]	Filter by keyword in contents
+\\[howm-view-grep-in-contents]	Search within results
 \\[howm-view-sort]	Sort
 \\[howm-view-sort-reverse]	Reverse order
 \\[howm-view-dired]	Invoke Dired-X
 \\[describe-mode]	This help
-\\[riffle-kill-buffer]	Quit
+\\[howm-view-kill-buffer]	Quit
 "
+  (howm-view-contents-mode-body))
+
+(defun howm-view-contents-mode-body ()
 ;   (kill-all-local-variables)
   (make-local-variable 'font-lock-keywords)
-  (cheat-font-lock-mode howm-view-font-lock-silent)
+  (cheat-font-lock-mode)
   (let ((ck `((,howm-view-header-regexp (0 howm-view-hilit-face))))
         (sk (or (howm-view-font-lock-keywords)
                 *howm-view-font-lock-keywords*)))
@@ -268,7 +286,9 @@ key	binding
   (let ((m keymap))
 ;;     (define-key m "?" 'howm-view-help)
     (define-key m "f" 'howm-view-filter)
-    (define-key m "G" 'howm-view-filter-by-contents)
+    (define-key m "F" 'howm-view-actually-filter-by-contents)
+    (define-key m "K" 'howm-view-actually-filter-by-keyword-in-contents)
+    (define-key m "G" 'howm-view-grep-in-contents)
     (define-key m "S" 'howm-view-sort)
     (define-key m "R" 'howm-view-sort-reverse)
     (define-key m "q" 'howm-view-kill-buffer)
@@ -301,7 +321,8 @@ key	binding
          (r (riffle-summary name item-list ':howm
                            (howm-view-in-background-p))))
     (if (null r)
-        (message "No match")
+        (message "No match: \"%s\"" name)
+      (howm-view-expire-uniq)
       ;; We want to entry font-lock keywords even when background-p.
       (when *howm-view-font-lock-keywords*
         (setq howm-view-font-lock-keywords *howm-view-font-lock-keywords*)))
@@ -582,9 +603,20 @@ But I'm not sure for multi-byte characters on other versions of emacsen."
                                    table nil t)))
     (call-interactively (cdr (assoc command table)))))
 
-(defun howm-view-filter-uniq ()
+(defalias 'howm-view-filter-uniq #'howm-view-toggle-uniq)
+(defvar howm-view-uniq-previous nil)
+(make-variable-buffer-local 'howm-view-uniq-previous)
+(defun howm-view-toggle-uniq ()
   (interactive)
-  (howm-view-filter-doit #'howm-filter-items-uniq))
+  (if howm-view-uniq-previous
+      (howm-view-summary-rebuild howm-view-uniq-previous)
+    (let ((prev (howm-view-item-list)))
+      (howm-view-filter-doit #'howm-filter-items-uniq)
+      ;; need to set howm-view-uniq-previous AFTER rebuilding of
+      ;; the summary buffer because howm-view-expire-uniq is called in it.
+      (setq howm-view-uniq-previous prev))))
+(defun howm-view-expire-uniq ()
+  (setq howm-view-uniq-previous nil))
 
 (defun howm-view-filter-by-name (&optional remove-p regexp)
   (interactive "P")
@@ -603,6 +635,32 @@ But I'm not sure for multi-byte characters on other versions of emacsen."
                (funcall #',filter item-list ,r rmv-p))))
     (howm-view-filter-doit f remove-p)))
 
+(defun howm-view-filter-by-keyword-in-summary (&optional remove-p keyword)
+  (interactive "P")
+  (howm-view-filter-by-keyword-general #'howm-view-filter-by-summary
+                                       remove-p keyword))
+
+(defun howm-view-grep-keyword-in-contents (&optional remove-p keyword)
+  (interactive "P")
+  (howm-view-filter-by-keyword-general #'howm-view-grep-in-contents
+                                       remove-p keyword))
+
+(defun howm-view-actually-filter-by-keyword-in-contents (&optional remove-p keyword)
+  (interactive "P")
+  (howm-view-filter-by-keyword-general #'howm-view-actually-filter-by-contents
+                                       remove-p keyword))
+
+(defun howm-view-filter-by-keyword-general (f &optional remove-p keyword)
+  (let* ((k (or keyword (howm-completing-read-keyword)))
+         (aliases (if (howm-support-aliases-p)
+                      (howm-keyword-aliases k)
+                    k))
+         (regexp (if (listp aliases)
+                     (mapconcat 'regexp-quote aliases "\\|")
+                   (regexp-quote aliases)))
+         (howm-view-use-grep nil))  ;; necessary for "\\|"
+    (funcall f remove-p regexp)))
+
 (defun howm-view-filter-by-date (&optional remove-p)
   (interactive "P")
   (howm-view-filter-by-time-range #'howm-filter-items-by-date
@@ -647,14 +705,31 @@ But I'm not sure for multi-byte characters on other versions of emacsen."
                               (min ,end (length item-list))))))
     (howm-view-filter-doit f)))
 
-(defun howm-view-filter-by-contents (&optional remove-p regexp)
+;; keep misnamed functions that aren't really "filters" for backward compatibility
+(defalias 'howm-view-filter-by-contents 'howm-view-grep-in-contents)
+(defalias 'howm-view-filter-by-keyword-in-contents 'howm-view-grep-keyword-in-contents)
+
+(defun howm-view-grep-in-contents (&optional remove-p regexp)
+  ;; not "filter" but "search in result" actually for backward
+  ;; compatibility
   (interactive "P")
+  (howm-view-filter-by-contents-gen "Search in result (grep): "
+                                    nil remove-p regexp))
+
+(defun howm-view-actually-filter-by-contents (&optional remove-p regexp)
+  ;; just "filter" without modifying the items
+  (interactive "P")
+  (howm-view-filter-by-contents-gen "Regexp: "
+                                    t remove-p regexp))
+
+(defun howm-view-filter-by-contents-gen (prompt purely &optional remove-p regexp)
   (let ((r (or regexp (howm-view-filter-read-from-minibuffer
-                       "Search in result (grep): "
+                       prompt
                        remove-p))))
-    (if remove-p
-        (howm-view-remove-by-contents r)
-      (howm-view-search-in-result r))))
+    (cond (purely (howm-view-summary-rebuild
+                   (howm-filter-items-by-contents (howm-view-item-list) r remove-p)))
+          (remove-p (howm-view-remove-by-contents r))
+          (t (howm-view-search-in-result r)))))
 
 (howm-if-ver1dot3 nil
   (defcustom howm-view-search-in-result-correctly t
@@ -1056,12 +1131,12 @@ in ITEM-LIST belongs."
                                         &optional remove-match)
   "Select items in ITEM-LIST according to REFERENCE-ITEM-LIST.
 When REMOVE-MATCH is nil, return value is list of items i in ITEM-LIST
-which satisfy the condition \"there exists i' in REFERENCE-ITEM-LIST
-such that i and i' belong to same paragraph\" (case 1).
-When REMOVE-MATCH is non-nil and not the symbol 'with-rest',
+which satisfy the condition \"there exists i\\=' in REFERENCE-ITEM-LIST
+such that i and i\\=' belong to same paragraph\" (case 1).
+When REMOVE-MATCH is non-nil and not the symbol \\='with-rest\\=',
 return value is complement of the above list;
 list of items in ITEM-LIST which do not satisfy the above condition (case 2).
-When REMOVE-MATCH is the symbol 'with-rest',
+When REMOVE-MATCH is the symbol \\='with-rest\\=',
 return value is (A . B), where A is the return value of case 1 and
 B is items in REFERENCE-ITEM-LIST that do not match in case 1."
   ;; 
@@ -1126,6 +1201,7 @@ B is items in REFERENCE-ITEM-LIST that do not match in case 1."
 
 (defun howm-rangeset-belong-p (point rs)
   (or (null (cdr rs))
+      (null point) ;; for howm-view-actually-filter-by-contents [2025-07-20]
       (cl-member-if (lambda (pair)
                            (and (<= (car pair) point) (<= point (cdr pair))))
                          (cdr rs))))
@@ -1336,10 +1412,10 @@ which simply calls howm-sort-items-by-NAME."
 (defmacro howm-sort-items (evaluator comparer item-list
                                              &optional reverse-p)
   `(let* ((howm-view-s-i-comparer ,comparer)
-          (cmp (if reverse-p
+          (cmp (if ,reverse-p
                    (lambda (a b) (funcall howm-view-s-i-comparer b a))
                  howm-view-s-i-comparer)))
-     (howm-sort ,evaluator cmp item-list)))
+     (howm-sort ,evaluator cmp ,item-list)))
 
 ;; ;; generate the below aliases for howm-test080714
 ;; (let ((methods '("random" "name" "numerical-name" "date" "reverse-date"
@@ -1371,7 +1447,7 @@ which simply calls howm-sort-items-by-NAME."
   (howm-sort-items #'(lambda (dummy) (random)) #'< item-list reverse-p))
 
 (defun howm-sort-items-by-name (item-list &optional reverse-p)
-  (howm-sort-items #'howm-view-item-basename #'string< reverse-p))
+  (howm-sort-items #'howm-view-item-basename #'string< item-list reverse-p))
 
 (defun howm-sort-items-by-numerical-name (item-list &optional reverse-p)
   (howm-sort-items (lambda (i)
@@ -1379,7 +1455,7 @@ which simply calls howm-sort-items-by-NAME."
                                (if (string-match "^[0-9]+$" b)
                                    (string-to-number b)
                                  howm-infinity)))
-                           #'< reverse-p))
+                           #'< item-list reverse-p))
 
 (defvar howm-view-sort-by-date-ignore-regexp "^[a-zA-Z]")
 (defun howm-sort-items-by-date (item-list &optional reverse-p)
@@ -1512,8 +1588,9 @@ matched can be nil, single, or multi."
 (howm-defvar-risky howm-view-dired-ls-command "ls")
 (howm-defvar-risky howm-view-dired-ls-options '("-l"))
 
-(defun dired-virtual (dir)
-  (howm-inhibit-warning-in-compilation))
+(when (not (fboundp 'dired-virtual))
+  (defun dired-virtual (dir)
+    (howm-inhibit-warning-in-compilation)))
 
 (defun howm-view-dired ()
   (interactive)
@@ -1555,9 +1632,13 @@ RNAME must be relative name."
   (goto-char (point-min))
   ;; Raw call of `dired-get-filename' and `dired-next-line' causes
   ;; warnings in compilation.
-  (while (let ((c (howm-funcall-if-defined (dired-get-filename 'no-dir t))))
-           (not (and c (equal (file-relative-name c) rname))))
-    (howm-funcall-if-defined (dired-next-line 1))))
+  (let ((last-point (point)))
+    (while (let ((c (howm-funcall-if-defined (dired-get-filename 'no-dir t))))
+             (not (and c (equal (file-relative-name c) rname))))
+      (howm-funcall-if-defined (dired-next-line 1))
+      (when (<= (point) last-point)
+        (error "Failed howm-view-dired-goto."))
+      (setq last-point (point)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; shell
