@@ -1,6 +1,7 @@
+;;; -*- lexical-binding: nil; -*-
 ;;; riffle.el --- template of list browser with immediate preview
-;;; Copyright (C) 2004, 2005-2022
-;;;   HIRAOKA Kazuyuki <khi@users.osdn.me>
+;;; Copyright (C) 2004, 2005-2025
+;;;   HIRAOKA Kazuyuki <kakkokakko@gmail.com>
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -58,6 +59,29 @@ even if you delete other windows explicitly."
 ;; experimental [2008-05-23]
 (defvar riffle-keep-window nil)
 
+(defcustom riffle-frame-alist `((name . "*howm*"))
+  "Initial frame parameters when `howm-view-window-location' is set to \\='tab."
+  :type 'alist
+  :group 'howm-list-bufwin)
+(defcustom riffle-tab-name "*howm*"
+  "Tab name when `howm-view-window-location' is set to \\='tab."
+  :type 'string
+  :group 'howm-list-bufwin)
+
+(defcustom howm-view-window-location nil
+  "Use separate frame/tab for howm summary/contents buffers."
+  :type '(radio (const :tag "Default" nil)
+                (const :tag "Frame" frame)
+                (const :tag "Tab" tab))
+  :group 'howm-list-bufwin)
+
+(defcustom howm-view-close-frame/tab-on-exit nil
+  "Close the frame/tab when exiting from howm summary/contents buffers."
+  :type '(radio (const :tag "Default" nil)
+                (const :tag "Close" t)
+                (const :tag "Close if only howm buffers are displayed" howm-only))
+  :group 'howm-list-bufwin)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; internal variables and accessors
 
@@ -125,29 +149,93 @@ even if you delete other windows explicitly."
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; common
 
-(defcustom riffle-mode-hook nil
-  "Hook run at the end of function `riffle-mode'"
-  :type 'hook
-  :group 'howm-hook)
+(define-derived-mode riffle-mode text-mode "Riffle")
 
-(defvar riffle-mode-map nil)
-(put 'riffle-mode-map 'risky-local-variable t)
-(defvar riffle-mode-syntax-table (make-syntax-table))
-(defvar riffle-mode-abbrev-table nil)
+(defvar riffle-protected-localvar-prefixes
+  '("action-lock-" "howm-" "illusion-" "riffle-")
+  "Prefixes of buffer-local variable names that are protected from
+`kill-all-local-variables', which is called in `riffle-mode' through
+`fundamental-mode'.")
 
-(defun riffle-mode ()
-  "not yet"
-  (setq major-mode 'riffle-mode
-        mode-name "Riffle")
-  (use-local-map riffle-mode-map)
-  (set-syntax-table riffle-mode-syntax-table)
-  (define-abbrev-table 'riffle-mode-abbrev-table nil)
-  (run-hooks 'riffle-mode-hook))
+(defun riffle-mode-localvar-advice (orig &rest args)
+  "Recover buffer-local variables that match
+`riffle-protected-localvar-prefixes' after
+`kill-all-local-variables' is called in `fundamental-mode'."
+  (let ((vs (buffer-local-variables))
+        (ret (apply orig args)))
+    (mapc (lambda (pair)
+            (let* ((v (car pair))
+                   ;; See the document of `kill-all-local-variables'
+                   ;; for this property.
+                   (protected (get v 'permanent-local))
+                   (matched
+                    (cl-find-if (lambda (p) (string-prefix-p p (symbol-name v)))
+                                riffle-protected-localvar-prefixes)))
+              (when (and matched (not protected))
+                (set (make-local-variable v) (cdr pair)))))
+          vs)
+    ret))
+
+(advice-add 'riffle-mode :around #'riffle-mode-localvar-advice)
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;; frame/tab
+
+(defun riffle-initialize-frame ()
+  (let* ((name (cdr (assoc 'name riffle-frame-alist)))
+         (selected (and name
+                        (condition-case err
+                            (progn (select-frame-by-name name) t)
+                          (error nil)))))
+    (unless selected
+      (select-frame (make-frame riffle-frame-alist)))))
+(defun riffle-initialize-tab ()
+  (tab-bar-switch-to-tab riffle-tab-name))
+
+(defun riffle-summary (&rest args)
+  (when (and (eq howm-view-window-location 'tab)
+             (version< emacs-version "29.1"))
+    (if (yes-or-no-p "Not supported. Reset howm-view-window-location? ")
+        (setq howm-view-window-location nil)
+      (error "Emacs < 29.1 doesn't support tab for howm-view-window-location.")))
+  (let ((orig-frame (selected-frame)))
+    (unwind-protect
+        (let* ((initializer-table '((frame . riffle-initialize-frame)
+                                    (tab . riffle-initialize-tab)))
+               (initializer (cdr (assoc howm-view-window-location
+                                        initializer-table))))
+          (when initializer
+            (funcall initializer))
+          (apply #'riffle-summary-doit args))
+      (when (eq howm-view-window-location 'frame)
+        (when iigrep-show-what  ;; Dirty! (Peeks at an internal variable.)
+          (select-frame-set-input-focus orig-frame))))))
+
+(defun riffle-kill-buffer (&rest args)
+  (interactive)
+  (let* ((howm-window-p
+          (lambda (w)
+            (with-current-buffer (window-buffer w)
+              (or howm-mode
+                  (member major-mode '(howm-view-summary-mode
+                                       howm-view-contents-mode))))))
+         (delete-frame/tab-p
+          (cond ((eq howm-view-close-frame/tab-on-exit t) t)
+                ((eq howm-view-close-frame/tab-on-exit 'howm-only)
+                 (cl-every howm-window-p
+                           (window-list nil 'no-minibuf)))
+                (t nil))))
+    (apply #'riffle-kill-buffer-doit args)
+    (when delete-frame/tab-p
+      (cond ((eq howm-view-window-location 'frame)
+             (delete-frame))
+            ((eq howm-view-window-location 'tab)
+             (tab-bar-close-tab))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; summary
 
-(defun riffle-summary (&optional name item-list type background)
+(defun riffle-summary-doit (&optional name item-list type background)
   "Create summary buffer for NAME, ITEM-LIST, and TYPE.
 When NAME is nil, default values for them are selected.
 Created buffer is shown immediately as far as BACKGROUND is nil.
@@ -365,8 +453,7 @@ This function returns effective value of ITEM-LIST."
       (kill-buffer buf))
     (get-buffer-create bufname)))
 
-(defun riffle-kill-buffer ()
-  (interactive)
+(defun riffle-kill-buffer-doit ()
   (when (riffle-p)
     (let* ((s (riffle-summary-buffer))
            (c (riffle-contents-buffer))
@@ -458,6 +545,8 @@ snap://Info-mode/emacs#File Variables
       (let ((even-window-heights (if size
                                      nil
                                    even-window-heights))
+            ;; Skip all user options and force the default behavior.
+            (display-buffer-overriding-action display-buffer-fallback-action)
             ;; Don't split windows further even when
             ;; riffle-pop-to-buffer is called twice.
             (pop-up-windows nil))
@@ -510,7 +599,7 @@ snap://Info-mode/emacs#File Variables
 (let ((m riffle-summary-mode-map))
   (define-key m " " 'riffle-pop-or-scroll-other-window)
   (define-key m [backspace] 'scroll-other-window-down)
-  (define-key m "\C-h" 'scroll-other-window-down)
+  (define-key m (kbd "DEL") 'scroll-other-window-down)
   (define-key m "j" 'riffle-scroll-other-window)
   (define-key m "k" 'riffle-scroll-other-window-down)
   (define-key m "@" 'riffle-summary-to-contents)
@@ -526,7 +615,7 @@ snap://Info-mode/emacs#File Variables
 (let ((m riffle-contents-mode-map))
   (define-key m " " 'scroll-up)
   (define-key m [backspace] 'scroll-down)
-  (define-key m "\C-h" 'scroll-down)
+  (define-key m (kbd "DEL") 'scroll-down)
   (define-key m "j" 'riffle-scroll-up)
   (define-key m "k" 'riffle-scroll-down)
   (define-key m "@" 'riffle-contents-to-summary)
@@ -583,7 +672,7 @@ snap://Info-mode/emacs#File Variables
                             "SampleS"
   "Sample summary mode.
 key	binding
----	-------
+---	-------\\<riffle-sample-summary-mode-map>
 \\[next-line]	Next item
 \\[previous-line]	Previous item
 \\[riffle-pop-or-scroll-other-window]	Pop and scroll contents
@@ -605,7 +694,7 @@ key	binding
                             "SampleC"
   "Sample contents mode.
 key	binding
----	-------
+---	-------\\<riffle-sample-contents-mode>
 \\[next-line]	Next line
 \\[previous-line]	Previous line
 \\[scroll-up]	Scroll up
